#! /usr/bin/python
# python 2.7.3
# optional argument: time
#    number of days to look back from today (the default is 14)
#
# version 1.0 initial release
# version 1.1 refactor into funtion
#


from datetime import *
import os
import sys

# data object to store the customer usage
class Customer:    
    def __init__(self, _name):
        self.name  = _name
        self.datesToDurations = {}
        self.datesToLaunches = {}
        self.username = ""
        self.host = ""
        self.timeOut = 0
    
    def totalDurationInRange(self, dateRange):
        sum = timedelta()
        for dt in dateRange:
            if dt in self.datesToDurations:
                sum += self.datesToDurations[dt]
        return str(sum)

    def totalLaunchesInRange(self, dateRange):
        sum = 0
        for dt in dateRange:
            if dt in self.datesToLaunches:
                sum += self.datesToLaunches[dt]
        return str(sum)
    
    def checkOut(self, time):
        day = time.date()
        self.timeOut = time
        if day not in self.datesToLaunches:
            self.datesToLaunches[day] = 0
        self.datesToLaunches[day] += 1
    
    def checkIn(self, time):
        if self.timeOut == 0:
            return
        day = self.timeOut.date()
        if day not in self.datesToDurations:
            self.datesToDurations[day] = timedelta()
        self.datesToDurations[day] += (time - self.timeOut)
        self.timeOut = 0
        
    def identity(self):
        return self.username + "@" + self.host
    
# end Customer object


# this function is used in iterating over a date range
def daterange(start_date, end_date):
    for n in range(int ((end_date - start_date).days)):
        yield start_date + timedelta(n)


# utility function to generate the report
def generateReport(daysToReport=14):
    os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])))

    firstDate = date.today() - timedelta(days=daysToReport)
    #print "starting " + str(firstDate)
    
    # load file, possible old files too (log rotations: report_log_file_name.yyyy.mm.dd), also handle server restarts in the same file.
    if not os.path.isdir("Logs"):
        return "Error: No Logs to report"
    
    fileLines = []
    files = os.listdir('Logs')
    for fileName in files:
        if fileName == 'UsageReport':
            continue # skip this one until the end
        if fileName.startswith('UsageReport.'):
            dt = datetime.strptime(fileName[12:], '%Y.%m.%d').date()
            if dt > firstDate:
                file = open("Logs/"+fileName, 'r')
                fileLines.extend(file.readlines())
                file.close()
    # now load the current log
    if os.path.isfile("Logs/UsageReport"):
        file = open("Logs/UsageReport", 'r')
        fileLines.extend(file.readlines())
        file.close()
    

    nameMap = {} # permanent owner of the customer objects
    poolMap = {}  # transient index on customer pool numbers
    outMap = {}  # transient index on customer host names
    currentYear = 1900 # OUT and IN lines don't record the year as part of the date... have to get it elsewhere
    for line in fileLines:
        # make map of license pool numbers to customer names (might change with different log sessions or when new keys are added)
        if line.startswith("PRODUCT"):
            parsedName = line.split('"')[5]
            parsedPool = line.split(' ')[3]
            if parsedName not in nameMap:
                nameMap[parsedName] = Customer(parsedName)
            poolMap[parsedPool] = nameMap[parsedName]
            #print 'parsed PRODUCT for {} in pool {}'.format(parsedName, parsedPool)
            
        elif line.startswith("REREAD") or line.startswith("START"):
            poolMap.clear()
            #print 'parsed REREAD or START'
        
        elif len(line) > 13 and line[2] == '/' and line[5] == '/' and line[10] == ' ' and line[13]==':':
            currentYear = int(line[6:10])
            #print 'parsed YEAR as {}'.format(currentYear)
            
        # parse through log, for each one recording the date in use and the duration, if duration is known
        # add this information to the object created for each pool
        elif line.startswith("OUT"):
            fields = line.split(' ')
            parsedPool = fields[3]
            customer = poolMap[parsedPool]
            customer.username = fields[4]
            customer.host = fields[5]
            time = datetime.strptime(' '.join(fields[-2:]).rstrip(), '%m/%d %H:%M:%S').replace(year=currentYear)
            customer.checkOut(time)
            outMap[customer.identity()] = customer
            #print 'parsed OUT for pool {} identified by {}'.format(parsedPool, customer.identity())
            
        elif line.startswith("IN"):
            fields = line.split(' ')
            parsedUsername = fields[4]
            parsedHost = fields[5]
            identity = parsedUsername+"@"+parsedHost
            if identity in outMap:
                customer = outMap[identity]
                time = datetime.strptime(' '.join(fields[-2:]).rstrip(), '%m/%d %H:%M:%S').replace(year=currentYear)
                customer.checkIn(time)
                del outMap[identity]
                #print 'parsed IN for identity {}'.format(identity)
            #else:
                #print 'ignore IN for identity {}'.format(identity)

    # generate two reports, one sorted by user, one sorted by date
    # in two formats, html tables & plain text
    #  user report: name, expiration, which days used, durations
    #  date report: who used trial on this day and for how long
    userLine = '{:20} | {:^8} | {:^10}\n'
    userReport = userLine.format("Customer name", "Launches", "Time in use")
    userReport += '\n'
    dateLine = '{:10} | {:20} | {:^8} | {:^10}\n'
    dateReport = {}
    for name, customer in nameMap.items():
        userReport += userLine.format(customer.name,
                                      customer.totalLaunchesInRange(daterange(firstDate, date.today())),
                                      customer.totalDurationInRange(daterange(firstDate, date.today())))
        for dt in daterange(firstDate, date.today()):
            launches = '?'
            duration = '?'
            include = False
            if dt not in dateReport:
                dateReport[dt] = dateLine.format(str(dt), "--------------------", "-----", "-------")
            if dt in customer.datesToLaunches:
                launches = str(customer.datesToLaunches[dt]) 
                include = True
            if dt in customer.datesToDurations:
                duration = str(customer.datesToDurations[dt])
                include = True
            if include:
                dateReport[dt] += dateLine.format("", customer.name, launches, duration)

    retString = ""
    retString += "---------------------" + "\n"
    retString += "Report Period Totals" + "\n"
    retString += "---------------------" + "\n"
    retString += userReport + "\n"
    retString += "---------------------" + "\n"
    retString += "Daily Usage" + "\n"
    retString += "---------------------" + "\n"
    retString += dateLine.format("Date", "Customer name", "Launches", "Time in use") + "\n"
    for d in sorted(dateReport, key=dateReport.get):
         retString += dateReport[d]

    return retString

# main will only be called if this file is run directly, not when included
def main():
    # arguments
    if len(sys.argv) > 1:
        report = generateReport(int(sys.argv[1]))
    else:
        report = generateReport()

    print report

if __name__ == "__main__":
    main()

